package boot;

import org.q.app.ui.MainFrameModule;
import org.q.toolkit.lang.LogTemplate;
import org.q.toolkit.swing.SwingThread;

import com.google.inject.Guice;

/**
 * Boots the application in the Swing Thread. The Swing Thread is used because all interactions with swing components
 * should be performed from the Swing event Thread or unexpected behavior might occur.
 * 
 * @author Erik Lievaart
 */
class Boot {

	static void runInSwingThread(final BootSequence work) throws Exception {
		SwingThread.invokeAndWait(new Runnable() {
			@Override
			public void run() {
				try {
					long start = System.currentTimeMillis();
					LogTemplate log = new LogTemplate("org.q.boot");
					log.info("Initializing Q");

					work.boot(Guice.createInjector(new MainFrameModule()));

					log.info("Startup complete in % ms", (System.currentTimeMillis() - start));
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
			}
		});

	}

}
