package boot;

import org.q.app.engine.script.ScriptInvoker;

import com.google.inject.Injector;

/**
 * Boot the application in a custom manner. This class makes it possible customize the application boot process. This is
 * useful for tests, but also allows a custom UI, for example, headless. Alternatively, this method might be used to
 * invoke a single engine operation without booting the UI.
 * 
 * @author Erik Lievaart
 */
public class Custom {
	private Custom() {
	}

	/**
	 * Main method.
	 * 
	 * @param scripts
	 *            Beanshell scripts to run in this custom boot process.
	 * @throws Exception
	 *             if it occurs during bootup.
	 */
	public static void main(final String... scripts) throws Exception {
		Boot.runInSwingThread(new BootSequence() {
			@Override
			public void boot(final Injector injector) {
				runScripts(injector.getInstance(ScriptInvoker.class), scripts);
			}
		});
	}

	private static void runScripts(final ScriptInvoker runner, final String... scripts) {
		runner.createEngine();
		for (String script : scripts) {
			runner.runResource(script);
		}
	}
}