package boot;

import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.script.ScriptException;

import org.q.app.config.ConfigurableResources;
import org.q.app.config.ResourcePaths;
import org.q.app.engine.script.ScriptInvoker;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.CollectionCheck;

import com.google.inject.Injector;

/**
 * Boot the application normally.
 * 
 * @author Erik Lievaart
 */
public class Main {
	private Main() {
	}

	/**
	 * Boot the application by running all scripts in the init and the q/init
	 * directories.
	 * 
	 * @param args
	 *            ignored.
	 * @throws Exception
	 *             if it occurs during bootup.
	 */
	public static void main(final String[] args) throws Exception {
		Boot.runInSwingThread(new BootSequence() {
			@Override
			public void boot(final Injector injector) throws IOException, ScriptException {
				init(injector);
			}
		});
	}

	private static void init(final Injector injector) throws IOException, ScriptException {
		ConfigurableResources resources = injector.getInstance(ConfigurableResources.class);

		List<String> scripts = resources.listAll(UrlTool.append(ResourcePaths.SCRIPT_DIR, "q/init"));
		scripts.addAll(resources.listAll(UrlTool.append(ResourcePaths.SCRIPT_DIR, "init")));

		CollectionCheck.notEmpty(scripts, "Fatal Error! Unable to boot, no script files found in init directories!");
		ScriptInvoker runner = injector.getInstance(ScriptInvoker.class);
		runner.createEngine();
		for (String script : scripts) {
			runner.runFile(new File(resources.getAbsolutePath(script)));
		}
	}

}
