package org.q.app.engine.concurrent;

import java.util.Map;

import org.q.app.config.ResourcePaths;
import org.q.app.engine.script.ScriptConfig;
import org.q.app.engine.script.ScriptInvocation;
import org.q.app.engine.script.SwingScript;
import org.q.toolkit.lang.collection.MapTool;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

class CommandQueueScripts {
	private static final String COMMAND_VAR = "command";
	private static final String ID_VAR = "id";
	private static final String MSG_VAR = "message";

	@Inject
	private SwingScript runner;

	public void add(final long id, final String label) {
		ScriptConfig config = new ScriptConfig(MapTool.of(ID_VAR, id, COMMAND_VAR, label));
		ScriptInvocation invocation = new ScriptInvocation(ResourcePaths.QUEUE_ADD_SCRIPT);
		runner.invoke(config, invocation);
	}

	public void done(final long id, final String label) {
		ScriptConfig config = new ScriptConfig(MapTool.of(ID_VAR, id, COMMAND_VAR, label));
		ScriptInvocation invocation = new ScriptInvocation(ResourcePaths.QUEUE_DONE_SCRIPT);
		runner.invoke(config, invocation);
	}

	public boolean failed(final long id, final String label, final String errorMessage) {
		Map<String, Object> variables = NewCollection.mapNotNull();

		variables.put(ID_VAR, id);
		variables.put(COMMAND_VAR, label);
		variables.put(MSG_VAR, errorMessage);

		ScriptInvocation i = new ScriptInvocation(ResourcePaths.UI_POPUP_SCRIPT, ResourcePaths.QUEUE_FAILED_SCRIPT);
		i.setCode("return result;");
		return runner.invoke(new ScriptConfig(variables), i);
	}

}
