package org.q.app.engine.concurrent;

import org.q.app.contract.ThreadPolicy;
import org.q.app.engine.impl.PluginRunnable;
import org.q.toolkit.lang.AssertionException;
import org.q.toolkit.swing.SwingThread;

import com.google.inject.Inject;
import com.google.inject.Singleton;

/**
 * The CommandScheduler schedules ShellCommands for execution.
 * 
 * @author Erik Lievaart
 */
@Singleton
public class CommandScheduler {

	@Inject
	private CommandQueue queue;
	@Inject
	private SingleThreadQueue single;

	/**
	 * Schedule a job for execution.
	 * 
	 * @param runnable
	 *            Job to schedule.
	 * @param policy
	 *            Threading policy to use. This policy determines if the job will be executed immediately or queued for
	 *            execution.
	 */
	public void schedule(final PluginRunnable runnable, final ThreadPolicy policy) {
		switch (policy) {

			case CURRENT:
				runnable.run();
				return;

			case SWING:
				SwingThread.invokeLater(runnable);
				return;

			case FORK:
				new Thread(runnable).start();
				return;

			case QUEUE:
				queue.invokeLater(runnable);
				return;

			case SINGLE:
				single.invokeLater(runnable);
				return;
		}
		throw new AssertionException("Unknown policy: %", policy);
	}

}
