package org.q.app.engine.impl;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import com.google.inject.BindingAnnotation;

/**
 * Interface for checking and invoking ShellCommands. This is one of the central interfaces in the application. Engines
 * can be injected through Guice using the {@link Core} or {@link Templated} annotations.
 * 
 * @author Erik Lievaart
 */
public interface Engine {

	/**
	 * Invoke a ShellCommand.
	 * 
	 * @param raw
	 *            ShellCommand in raw form.
	 */
	public void invoke(final String raw);

	/**
	 * Guice annotation for the core Engine. Annotate a Guice injected {@link Engine} field with this annotation to
	 * inject the core engine. The core engine invokes Q commands literally.
	 * 
	 * @author Erik Lievaart
	 */
	@Retention(RetentionPolicy.RUNTIME)
	@Target({ ElementType.FIELD, ElementType.PARAMETER })
	@BindingAnnotation
	public @interface Core {
	}

	/**
	 * Guice annotation for the templated Engine. Annotate a Guice injected {@link Engine} field with this annotation to
	 * inject the template engine. The template engine applies engine templates before invoking the Q commands.
	 * 
	 * @author Erik Lievaart
	 */
	@Retention(RetentionPolicy.RUNTIME)
	@Target({ ElementType.FIELD, ElementType.PARAMETER })
	@BindingAnnotation
	public @interface Templated {
	}

}
