package org.q.app.engine.impl;

import org.q.app.contract.PluginContext;
import org.q.app.engine.parse.ShellCommand;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.IdGenerator;
import org.q.toolkit.lang.Str;
import org.q.toolkit.test.AntProperties;

import com.google.inject.Inject;

/**
 * An excutable version of a ShellCommand. Calling the run method will execute
 * the ShellCommand immediately
 * 
 * @author Erik Lievaart
 */
public class PluginRunnable implements Runnable {

	private static final IdGenerator GENERATOR = new IdGenerator();

	private final long id = GENERATOR.next();
	private boolean error = false;
	private String errorMessage = Str.EMPTY;
	private String label;
	private ShellCommand command;

	@Inject
	private PluginContext context;
	@Inject
	private PluginRunner runner;

	// @Inject
	// private ErrorScript errorScript;

	PluginRunnable() {
	}

	void setShellCommand(final ShellCommand command) {
		this.command = command;
	}

	/**
	 * Invoke the ShellCommand.
	 */
	@Override
	public void run() {
		try {
			Check.notNull(command, "ShellCommand not set!");
			runner.invoke(command, context);

		} catch (Exception e) {
			if (AntProperties.isTestMode()) {
				throw new RuntimeException(e);
			}
			error = true;
			errorMessage = e.getMessage();
			e.printStackTrace();
		}
	}

	/**
	 * Get the display label.
	 */
	public String getLabel() {
		return label;
	}

	void setLabel(final String raw) {
		this.label = raw;
	}

	/**
	 * Get the error flag.
	 * 
	 * @return true if the error flag was set. Until the run method is invoked,
	 *         this method always returns false.
	 */
	public boolean isError() {
		return error;
	}

	public String getErrorMessage() {
		return errorMessage;
	}

	/**
	 * Get the name of the plugin to invoke.
	 */
	public String getCommandName() {
		return command.getName();
	}

	public long getId() {
		return id;
	}

}
