package org.q.app.ui.bind.menu;

import java.awt.event.KeyEvent;

import javax.swing.JMenu;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.q.toolkit.reflect.FieldTool;

// use only as prototype
/**
 * Factory class for building a {@link JMenu}. This class should be used as a prototype.
 * 
 * @author Erik Lievaart
 */
public class JMenuBuilder implements ComponentBuilder<JMenu> {

	private final JMenu instance = new JMenu();

	/**
	 * Set the label of the {@link JMenu}
	 */
	public void setText(final String text) {
		instance.setText(text);
	}

	/**
	 * Set the mnemonic of the {@link JMenu}.
	 * 
	 * @param text
	 *            The name of a constant on the KeyEvent class.
	 */
	public void setMnemonic(final String text) {
		instance.setMnemonic((Integer) FieldTool.getConstant(KeyEvent.class, text));
	}

	@Override
	public void addChild(final ComponentBuilder<?> factory) {
		instance.add(factory.instance());
	}

	@Override
	public JMenu instance() {
		return instance;
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this);
	}
}
