package org.q.app.ui.render.browser;

import java.util.List;

import org.q.app.vfs.VirtualFile;
import org.q.app.vfs.VirtualFiles;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.collection.NewCollection;

/**
 * Wraps a VirtualFile and gives it a toString method that can be shown in the file browser windows.
 * 
 * @author Erik Lievaart
 */
public class VirtualFileWrapper {

	private final VirtualFile file;

	/**
	 * Wrap a VirtualFile.
	 */
	public VirtualFileWrapper(final VirtualFile file) {
		Check.notNull(file);

		this.file = file;
	}

	/**
	 * Sort and list the children.
	 * 
	 * @param hidden
	 *            toggle hidden files.
	 * @return {@link VirtualFileWrapper}'s for the children.
	 */
	public VirtualFileWrapper[] list(final boolean hidden) {
		List<VirtualFileWrapper> result = NewCollection.list();

		for (VirtualFile child : VirtualFiles.getChildrenAlphabetically(file)) {
			if (hidden || !child.isHidden()) {
				result.add(new VirtualFileWrapper(child));
			}
		}
		return result.toArray(new VirtualFileWrapper[] {});
	}

	/**
	 * Get the contained {@link VirtualFile}.
	 */
	public VirtualFile getVirtualFile() {
		return file;
	}

	@Override
	public String toString() {
		if (!file.isLocal()) { // for performance
			return file.getName();
		}
		return file.exists() ? file.getName() : "????";
	}

}
