package org.q.plugins.new_;

import java.io.IOException;

import org.apache.commons.io.IOUtils;
import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.FileSystemException;
import org.q.app.contract.Invocable;
import org.q.app.contract.Swallows;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.ui.context.BrowseEvents;
import org.q.toolkit.lang.Check;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.FORK)
@Doc("Create files and directories in any writable filesystem (note: can create nested paths)")
class NewShellCommand implements Invocable, Swallows {
	@Inject
	private BrowseEvents events;

	private FileObject dir;
	private String name;
	private Boolean file;

	@Flag(group = "main", values = { "$dir", "`file`" })
	@Doc("Create a file. Two arguments: 1) destination dir 2) file name")
	public NewShellCommand file(final FileObject dir, final String name) {
		this.dir = dir;
		this.name = name;
		file = Boolean.TRUE;
		return this;
	}

	@Flag(group = "main", values = { "$dir", "`directory`" }, primary = true)
	@Doc("Create a directory. Two arguments: 1) destination dir 2) directory name")
	public NewShellCommand directory(final FileObject dir, final String name) {
		this.dir = dir;
		this.name = name;
		file = Boolean.FALSE;
		return this;
	}

	@Override
	public void invoke() throws IOException {
		if (file) {
			invoke(null);
		} else {
			dir.resolveFile(name).createFolder();
		}
		if (events != null) { // if necessary for tests
			events.refresh();
		}
	}

	@Override
	@Doc("Contents of the file to create (only works for files).")
	public void invoke(final String raw) throws IOException {
		FileObject create = dir.resolveFile(name);
		create.createFile();

		if (raw != null) {
			IOUtils.write(raw, create.getContent().getOutputStream());
			create.close();
		}
	}

	@Override
	public void validate() throws ValidationException {
		try {
			Check.isTrue(dir.isWriteable(), "Directory not writable %", dir);

		} catch (FileSystemException e) {
			throw new ValidationException("unable to access file system: " + e.getMessage(), e);
		}
	}

	@Override
	public void validate(final String raw) throws ValidationException {
		Check.isTrue(file, "Can only swallow with -file flag");
	}
}
