package org.q.plugins.open;

import java.io.IOException;

import org.q.app.config.ResourcePaths;
import org.q.app.contract.Invocable;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.engine.script.ScriptInvocation;
import org.q.app.engine.script.SwingScript;
import org.q.app.vfs.VirtualFile;
import org.q.app.vfs.VirtualFiles;
import org.q.toolkit.lang.Check;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.SINGLE)
@Doc("Open m3u files")
class OpenShellCommand implements Invocable {

	private VirtualFile file;

	@Inject
	private SwingScript script;
	@Inject
	private M3uSelection selection;

	@Flag(group = "main", values = { "$url" }, primary = true)
	@Doc("Open a m3u file. Argument: m3u file")
	public OpenShellCommand m3u(final VirtualFile file) {
		this.file = file;
		return this;
	}

	@Override
	public void invoke() throws IOException {
		selection.setup(M3u.parse(file.getParentFile(), VirtualFiles.toString(file, "ISO8859_1")));
		script.invoke(new ScriptInvocation(ResourcePaths.UI_M3U));
	}

	@Override
	public void validate() throws ValidationException {
		Check.isTrue(file.exists(), "File does not exist %", file);
		Check.isEqual(file.getExtension(), "m3u", "File does not have the m3u extension: %", file.getUrl());
	}

}
