package org.q.plugins.replace;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.q.app.contract.Swallows;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.contract.annotation.ValidateSwallowed;
import org.q.app.engine.impl.Engine;
import org.q.app.engine.impl.Engine.Core;
import org.q.app.engine.impl.InputExaminer;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.CURRENT)
@ValidateSwallowed
@Doc("Replace property holders with variables")
class ReplaceShellCommand implements Swallows {
	private final Log log = LogFactory.getLog(getClass());

	private boolean dump = false;

	@Inject
	@Core
	private Engine engine;
	@Inject
	private TokenReplacer tokens;
	@Inject
	private InputExaminer examiner;

	@Flag(values = "`,`")
	@Doc("Convert spaces into the specified separator. Will convert spaces between URL's, but not spaces in URL's")
	public ReplaceShellCommand separator(final String separator) {
		tokens.replaceWhitespace(separator);
		return this;
	}

	@Flag(values = { "` `", "`\\bw`" })
	@Doc("Replace string occurences in URL's. 2 arguments: 1) string to replace 2) replacement string.")
	public ReplaceShellCommand replace(final String from, final String to) {
		tokens.escapeSubstring(from, to);
		return this;
	}

	@Flag
	@Doc("Do not execute, dump replaced swallowed content. Requires Q to be opened from a terminal window.")
	public ReplaceShellCommand log() {
		dump = true;
		return this;
	}

	@Flag
	@Doc("Remove url protocol and return only the path. For example: 'file:///tmp' => '/tmp'")
	public ReplaceShellCommand path() {
		tokens.removeProtocol();
		return this;
	}

	@Flag
	@Doc("Escape URL escape characters. For example: '@' => '%40'")
	public ReplaceShellCommand url() {
		tokens.replaceUrlCharacters();
		return this;
	}

	@Override
	@Doc("Replace placeholders of the format ${name} with their values")
	public void invoke(final String raw) throws Exception {
		String replaced = tokens.replace(raw.trim());

		if (dump) {
			log.info(replaced);
		} else {
			engine.invoke(replaced);
		}
	}

	@Override
	public void validate(final String raw) throws ValidationException {
		examiner.examineRaw(raw.trim());
	}

}
