package org.q.toolkit.io;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.FormattedException;
import org.q.toolkit.lang.collection.ValidatingMap;

public class ResourceTool {

	private static final String NULL_RESOURCE_MESSAGE = "<null> is not a valid resource";

	public static ValidatingMap<String, String> loadProperties(String res) {
		checkResourceString(res);
		try {
			InputStream is = ResourceTool.getInputStream(res);
			Check.notNull(is, "% not found", res);

			return new ValidatingMap(res, PropertiesIO.load(is));

		} catch (IOException e) {
			throw new FormattedException("% is not a valid properties file; %", e, e.getMessage());
		}
	}

	public static InputStream getInputStream(String res) {
		checkResourceString(res);
		try {
			InputStream stream = getLiteral().getResourceAsStream(res);
			Check.notNull(stream, NULL_RESOURCE_MESSAGE);
			return stream;
		} catch (Exception e) {
			throw new FormattedException("% could not be read; %", e, res, e.getMessage());
		}
	}

	public static URL getURL(String res) {
		checkResourceString(res);
		return getLiteral().getResource(res);
	}

	private static void checkResourceString(String res) {
		Check.matches(res, "/.*", "% resource path must start with a slash", res);
	}

	private static Class<?> getLiteral() {
		return ResourceTool.class;
	}

}
